package server

import (
	"context"
	"fmt"
	"net"
	"time"

	"github.com/prometheus/client_golang/prometheus"
	log "github.com/sirupsen/logrus"

	cpb "code.fbi.h-da.de/danet/api/go/gosdn/csbi"
	"code.fbi.h-da.de/danet/gosdn/metrics"
	"code.fbi.h-da.de/danet/gosdn/store"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/peer"
	"google.golang.org/grpc/status"
)

type csbi struct {
	cpb.UnimplementedCsbiServer
}

func (s csbi) Hello(ctx context.Context, syn *cpb.Syn) (*cpb.Ack, error) {
	labels := prometheus.Labels{"service": "csbi", "rpc": "hello"}
	start := metrics.StartHook(labels, grpcRequestsTotal)
	defer metrics.FinishHook(labels, start, grpcRequestDurationSecondsTotal, grpcRequestDurationSeconds)
	ch, err := pndc.PendingChannels(store.FromString(syn.Id))
	if err != nil {
		return nil, handleRPCError(labels, err)
	}
	p, ok := peer.FromContext(ctx)
	if !ok {
		e := fmt.Errorf("no peer information in context %v", ctx)
		return nil, handleRPCError(labels, e)
	}
	csbiAddress, err := removePort(p.Addr)
	if err != nil {
		log.Error(err)
		return nil, status.Errorf(codes.Aborted, "%v", err)
	}
	ch <- store.DeviceDetails{
		ID:      syn.Id,
		Address: net.JoinHostPort(csbiAddress, syn.Address),
	}
	details := <-ch
	log.Infof("ack to csbi %v", syn.Id)
	log.WithField("transport option", details.TransportOption).Debug("csbi ack transport options")
	return &cpb.Ack{
		Timestamp:       time.Now().UnixNano(),
		TransportOption: details.TransportOption,
	}, nil
}

func removePort(ip net.Addr) (string, error) {
	addr, ok := ip.(*net.TCPAddr)
	if !ok {
		return "", fmt.Errorf("invalid type assertion")
	}
	return addr.IP.String(), nil
}
