package nucleus

import (
	tpb "code.fbi.h-da.de/danet/api/go/gosdn/transport"
	"code.fbi.h-da.de/danet/gosdn/interfaces/southbound"
	"code.fbi.h-da.de/danet/gosdn/interfaces/transport"
	"code.fbi.h-da.de/danet/gosdn/nucleus/errors"
)

// NewTransport receives TransportOptions and returns an appropriate Transport
// implementation
func NewTransport(opts *tpb.TransportOption, sbi southbound.SouthboundInterface) (transport.Transport, error) {
	if opts == nil {
		return nil, &errors.ErrInvalidParameters{
			Func:  NewTransport,
			Param: "'opt' cannot be 'nil'",
		}
	}
	if sbi == nil {
		return nil, errors.ErrInvalidParameters{
			Param: "'sbi' cannot be 'nil'",
		}
	}
	if !validTransportOptions(opts) {
		return nil, &errors.ErrInvalidTransportOptions{Opt: opts}
	}
	switch o := opts.TransportOption.(type) {
	case *tpb.TransportOption_GnmiTransportOption:
		return newGnmiTransport(opts, sbi)
	default:
		return nil, &errors.ErrInvalidTransportOptions{Opt: o}
	}
}

func validTransportOptions(opt *tpb.TransportOption) bool {
	return opt.GetGnmiTransportOption() != nil ||
		opt.GetRestconfTransportOption() != nil
}
