package store

import (
	"code.fbi.h-da.de/danet/gosdn/interfaces/change"
	"code.fbi.h-da.de/danet/gosdn/nucleus/errors"
	"github.com/google/uuid"
	log "github.com/sirupsen/logrus"

	ppb "code.fbi.h-da.de/danet/api/go/gosdn/pnd"
)

// ChangeStore is used to store Changes
type ChangeStore struct {
	*genericStore
}

// NewChangeStore returns a ChangeStore
func NewChangeStore() *ChangeStore {
	return &ChangeStore{genericStore: newGenericStore()}
}

// GetChange takes a Change's UUID and returns the Change. If the requested
// Change does not exist an error is returned.
func (s *ChangeStore) GetChange(id uuid.UUID) (change.Change, error) {
	item, err := s.genericStore.Get(id)
	if err != nil {
		return nil, err
	}
	change, ok := item.(change.Change)
	if !ok {
		return nil, &errors.ErrInvalidTypeAssertion{
			Value: change,
			Type:  "change",
		}
	}
	log.WithFields(log.Fields{
		"uuid": id,
	}).Debug("change was accessed")
	return change, nil
}

// Pending returns the UUIDs of all pending changes
func (s *ChangeStore) Pending() []uuid.UUID {
	return filterChanges(s, ppb.Change_PENDING)
}

// Committed returns the UUIDs of all pending changes
func (s *ChangeStore) Committed() []uuid.UUID {
	return filterChanges(s, ppb.Change_COMMITTED)
}

// Confirmed returns the UUIDs of all pending changes
func (s *ChangeStore) Confirmed() []uuid.UUID {
	return filterChanges(s, ppb.Change_CONFIRMED)
}

func filterChanges(store *ChangeStore, state ppb.Change_State) []uuid.UUID {
	changes := make([]uuid.UUID, 0)
	for _, ch := range store.Store {
		switch change := ch.(type) {
		case change.Change:
			if change.State() == state {
				changes = append(changes, change.ID())
			}
		}
	}
	return changes
}
