package store

import (
	"reflect"
	"sync"
	"testing"

	"code.fbi.h-da.de/danet/gosdn/interfaces/networkdomain"
	"code.fbi.h-da.de/danet/gosdn/interfaces/store"
	"code.fbi.h-da.de/danet/gosdn/mocks"
	"github.com/google/uuid"
)

func Test_pndStore_get(t *testing.T) {
	pndMock := &mocks.NetworkDomain{}
	pndMock.On("ID").Return(defaultPndID)

	type fields struct {
		genericStore *genericStore
	}
	type args struct {
		id uuid.UUID
	}
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    networkdomain.NetworkDomain
		wantErr bool
	}{
		{
			name: "exists",
			fields: fields{
				&genericStore{
					Store: map[uuid.UUID]store.Storable{
						defaultPndID: pndMock,
					},
					storeLock: sync.RWMutex{},
				},
			},
			args:    args{id: defaultPndID},
			want:    pndMock,
			wantErr: false,
		},
		{
			name: "fails",
			fields: fields{
				&genericStore{
					Store: map[uuid.UUID]store.Storable{
						defaultPndID: pndMock,
					},
					storeLock: sync.RWMutex{},
				},
			},
			args:    args{id: iid},
			wantErr: true,
		},
		{
			name: "fails empty",
			fields: fields{
				genericStore: &genericStore{},
			},
			args:    args{id: defaultPndID},
			wantErr: true,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			s := PndStore{
				genericStore: tt.fields.genericStore,
			}
			got, err := s.Get(tt.args.id)
			if (err != nil) != tt.wantErr {
				t.Errorf("get() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if !reflect.DeepEqual(got, tt.want) {
				t.Errorf("get() got = %v, want %v", got, tt.want)
			}
		})
	}
}
