package store

import (
	"code.fbi.h-da.de/danet/gosdn/interfaces/southbound"
	"code.fbi.h-da.de/danet/gosdn/nucleus/errors"

	"github.com/google/uuid"
	log "github.com/sirupsen/logrus"
)

// SbiStore is used to store SouthboundInterfaces
type SbiStore struct {
	*genericStore
}

// NewSbiStore returns a SbiStore
func NewSbiStore() *SbiStore {
	return &SbiStore{genericStore: newGenericStore()}
}

// GetSBI takes a SouthboundInterface's UUID and returns the SouthboundInterface. If the requested
// SouthboundInterface does not exist an error is returned.
func (s *SbiStore) GetSBI(id uuid.UUID) (southbound.SouthboundInterface, error) {
	item, err := s.Get(id)
	if err != nil {
		return nil, err
	}
	sbi, ok := item.(southbound.SouthboundInterface)
	if !ok {
		return nil, &errors.ErrInvalidTypeAssertion{
			Value: sbi,
			Type:  "SouthboundInterface",
		}
	}
	log.WithFields(log.Fields{
		"uuid": id,
	}).Debug("southbound interface was accessed")
	return sbi, nil
}
