package store

import (
	"reflect"
	"sync"
	"testing"

	"code.fbi.h-da.de/danet/gosdn/interfaces/southbound"
	"code.fbi.h-da.de/danet/gosdn/interfaces/store"
	"code.fbi.h-da.de/danet/gosdn/mocks"

	"github.com/google/uuid"
)

func Test_sbiStore_get(t *testing.T) {
	openConfig := &mocks.SouthboundInterface{}
	openConfig.On("ID").Return(defaultSbiID)

	type fields struct {
		genericStore *genericStore
	}
	type args struct {
		id uuid.UUID
	}
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    southbound.SouthboundInterface
		wantErr bool
	}{
		{
			name: "exists",
			fields: fields{
				&genericStore{
					Store: map[uuid.UUID]store.Storable{
						defaultSbiID: openConfig,
					},
					storeLock: sync.RWMutex{},
				},
			},
			args:    args{id: defaultSbiID},
			want:    openConfig,
			wantErr: false,
		},
		{
			name: "fails",
			fields: fields{
				&genericStore{
					Store: map[uuid.UUID]store.Storable{
						defaultSbiID: openConfig,
					},
					storeLock: sync.RWMutex{},
				},
			},
			args:    args{id: iid},
			wantErr: true,
		},
		{
			name: "fails empty",
			fields: fields{
				genericStore: &genericStore{},
			},
			args:    args{id: defaultSbiID},
			wantErr: true,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			s := SbiStore{genericStore: tt.fields.genericStore}

			got, err := s.Get(tt.args.id)
			if (err != nil) != tt.wantErr {
				t.Errorf("get() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if !reflect.DeepEqual(got, tt.want) {
				t.Errorf("get() got = %v, want %v", got, tt.want)
			}
		})
	}
}
